-- Status: 20.07.2016

------------------------------------------------------------
--- BUILD BASIC TABLE FOR LANGUAGE AND CLEAN SO-URL DATA ---
------------------------------------------------------------
SELECT
  id,
  CASE
    WHEN REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/a\/[\d]+)')
    THEN REGEXP_EXTRACT(url, r'(https?:\/\/stackoverflow\.com\/a\/[\d]+)')
    WHEN REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/q\/[\d]+)')
    THEN REGEXP_EXTRACT(url, r'(https?:\/\/stackoverflow\.com\/q\/[\d]+)')
    WHEN REGEXP_MATCH(url, r'https?:\/\/stackoverflow\.com\/questions\/[\d]+\/[^\s\/\#]+(?:\/|\#)([\d]+)')
    THEN CONCAT("http://stackoverflow.com/a/", REGEXP_EXTRACT(url, r'https?:\/\/stackoverflow\.com\/questions\/[\d]+\/[^\s\/\#]+(?:\/|\#)([\d]+)'))
    WHEN REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/questions\/[\d]+)')
    THEN CONCAT("http://stackoverflow.com/q/", REGEXP_EXTRACT(url, r'https?:\/\/stackoverflow\.com\/questions\/([\d]+)'))
    ELSE url END as url,
    line,
  repo_name,
  path
FROM (
  SELECT
    id,
    REGEXP_REPLACE(
      REGEXP_EXTRACT(LOWER(line), r'(https?://stackoverflow\.com/[^\s)\.\"]*)'),
      r'(^https)',
      'http'
    ) as url,
    line,
    repo_name,
    path
  FROM (
    SELECT
      f.id as id,
      SPLIT(content, '\n') line,
      repo_name,
      path
    FROM (
      SELECT
        id,
        repo_name,
        path
      FROM
        [bigquery-public-data:github_repos.files]
      WHERE
        LOWER(path) LIKE '%.c'
        OR LOWER(path) LIKE '%.cpp'
        OR LOWER(path) LIKE '%.cc'
        OR LOWER(path) LIKE '%.cxx'
        OR LOWER(path) LIKE '%.c++'
        OR LOWER(path) LIKE '%.h'
        OR LOWER(path) LIKE '%.hh'
        OR LOWER(path) LIKE '%.hpp'
        OR LOWER(path) LIKE '%.hxx'
        OR LOWER(path) LIKE '%.h++'
    ) as f
    JOIN [bigquery-public-data:github_repos.contents] as c
    ON f.id = c.id
  )
  WHERE
    REGEXP_MATCH(line, r'(?i:https?://stackoverflow\.com/[^\s)\.\"]*)')
);

=> c



----------------------------------------------------
--- ONLY CONSIDER A AND Q LINKS IN THE FOLLOWING ---
----------------------------------------------------
SELECT
  id, url, line,
  repo_name, path,
  -- not all repos have master branch, thus link might be broken!
  CONCAT("https://github.com/", repo_name, "/blob/master/", path) as link
FROM [snippets.c]
WHERE
  REGEXP_MATCH(url, r'(http:\/\/stackoverflow\.com\/(?:a|q)\/[\d]+)');
  
=> c_a_q


-------------------------------------------------
--- STORE OTHER REFERENCES THAN A AND Q LINKS ---
-------------------------------------------------
SELECT
  id, url, line,
  repo_name, path,
  -- not all repos have master branch, thus link might be broken!
  CONCAT("https://github.com/", repo_name, "/blob/master/", path) as link
FROM [snippets.c]
WHERE
  NOT REGEXP_MATCH(url, r'(http:\/\/stackoverflow\.com\/(?:a|q)\/[\d]+)');
  
=> c_other


------------------------------------------------------------------------------------
--- CREATE TABLE WITH DISTINCT SO-URLS PER DISTINCT FILES (SAME FILE ID AND URL) ---
------------------------------------------------------------------------------------
SELECT
  id, url, line,
  -- not all repos have master branch, thus link might be broken!
  CONCAT("https://github.com/", repo_name, "/blob/master/", path) as link,
  repo_name, path
FROM (
  SELECT
    distinct_id_url_2.id as id,
    distinct_id_url_2.url as url,
    distinct_id_url_2.repo_name as repo_name,
    distinct_id_url_2.path as path,
    MIN(all_2.line) as line
  FROM (
    SELECT
      distinct_id_url.id as id,
      distinct_id_url.url as url,
      distinct_id_url.repo_name as repo_name,
      MIN(all.path) as path
      FROM (
        SELECT
          id, url, MIN(repo_name) as repo_name
        FROM
          [snippets.c_a_q]
        GROUP BY
          id, url
      ) as distinct_id_url
      LEFT JOIN [snippets.c_a_q] as all
        ON all.id = distinct_id_url.id
        AND all.url = distinct_id_url.url
        AND all.repo_name = distinct_id_url.repo_name 
    GROUP BY id, url, repo_name
  ) as distinct_id_url_2
  LEFT JOIN [snippets.c_a_q] as all_2
  ON all_2.id = distinct_id_url_2.id
    AND all_2.url = distinct_id_url_2.url
    AND all_2.repo_name = distinct_id_url_2.repo_name
    AND all_2.path = distinct_id_url_2.path
  GROUP BY id, url, repo_name, path
); 
  
=> c_a_q_distinct_url_file


------------------------------------------------------------------
---  !!!!! Problem: Copied and slightly modified files !!!!!   ---
------------------------------------------------------------------
-------------------------------------------------------------------------------
--- CREATE TABLE WITH DISTINCT LINES PER DISTINCT FILES (SAME LINE AND URL) ---
-------------------------------------------------------------------------------
SELECT
  url, line, -- removed id because single file is only example for combination of line and url
  -- not all repos have master branch, thus link might be broken!
  CONCAT("https://github.com/", repo_name, "/blob/master/", path) as link,
  repo_name,
  path
FROM (
  SELECT
    distinct_url_line_2.id as id,
    distinct_url_line_2.url as url,
    distinct_url_line_2.line as line,
    distinct_url_line_2.repo_name as repo_name,
    MIN(all_2.path) as path
  FROM (
      SELECT
        distinct_url_line.id as id,
        distinct_url_line.url as url,
        distinct_url_line.line as line,
        MIN(all.repo_name) as repo_name
      FROM (
          SELECT
            MIN(id) as id, url, line
          FROM
            [snippets.c_a_q]
          GROUP BY
            url, line
      ) as distinct_url_line
      LEFT JOIN [snippets.c_a_q] as all
      ON all.id = distinct_url_line.id
        AND all.url = distinct_url_line.url
        AND all.line = distinct_url_line.line
      GROUP BY
        id, url, line
  ) as distinct_url_line_2
  LEFT JOIN [snippets.c_a_q] as all_2
  ON all_2.id = distinct_url_line_2.id
    AND all_2.url = distinct_url_line_2.url
    AND all_2.line = distinct_url_line_2.line
    AND all_2.repo_name = distinct_url_line_2.repo_name
  GROUP BY
    id, url, line, repo_name
)

=> c_a_q_distinct_url_line


---------------------------------------------------------
---  !!!!! Map As to Qs using R Script before !!!!!   ---
---------------------------------------------------------
SELECT url
FROM [snippets.c_a_q]
WHERE REGEXP_MATCH(url, r'(http:\/\/stackoverflow\.com\/a\/[\d]+)')
GROUP BY url;

=> c_a_export.csv

-------------------------------------------------------------
--- EXPORT c_a_export.csv, IMPORT c_a_q_mapping.csv ---
-------------------------------------------------------------
=> c_a_q_mapping (columns url_a and url_b)

-------------------------------------------------------------------
--- CREATE TABLE WITH A URLS REPLACED BY Q URLS (complete data) ---
-------------------------------------------------------------------
SELECT 
  aq.id as id,
  CASE
    WHEN m.url_q is not null
    THEN m.url_q
    ELSE aq.url
  END as url,
  aq.line as line,
  aq.link as link,
  aq.repo_name as repo_name,
  aq.path as path
FROM [snippets.c_a_q] aq
LEFT JOIN [snippets.c_a_q_mapping] m
ON aq.url = m.url_a;

=> c_q


-----------------------------------------------------------------------
--- CREATE TABLE WITH A URLS REPLACED BY Q URLS (distinct file-url) ---
-----------------------------------------------------------------------
SELECT
  id, url, line,
  -- not all repos have master branch, thus link might be broken!
  CONCAT("https://github.com/", repo_name, "/blob/master/", path) as link,
  repo_name, path
FROM (
  SELECT
    distinct_id_url_2.id as id,
    distinct_id_url_2.url as url,
    distinct_id_url_2.repo_name as repo_name,
    distinct_id_url_2.path as path,
    MIN(all_2.line) as line
  FROM (
    SELECT
      distinct_id_url.id as id,
      distinct_id_url.url as url,
      distinct_id_url.repo_name as repo_name,
      MIN(all.path) as path
      FROM (
        SELECT
          id, url, MIN(repo_name) as repo_name
        FROM
          [snippets.c_q]
        GROUP BY
          id, url
      ) as distinct_id_url
      LEFT JOIN [snippets.c_q] as all
        ON all.id = distinct_id_url.id
        AND all.url = distinct_id_url.url
        AND all.repo_name = distinct_id_url.repo_name 
    GROUP BY id, url, repo_name
  ) as distinct_id_url_2
  LEFT JOIN [snippets.c_q] as all_2
  ON all_2.id = distinct_id_url_2.id
    AND all_2.url = distinct_id_url_2.url
    AND all_2.repo_name = distinct_id_url_2.repo_name
    AND all_2.path = distinct_id_url_2.path
  GROUP BY id, url, repo_name, path
); 
  
=> c_q_distinct_url_file


-----------------------------------------------------------------------
--- CREATE TABLE WITH A URLS REPLACED BY Q URLS (distinct line-url) ---
-----------------------------------------------------------------------
SELECT
  url, line, -- removed id because single file is only example for combination of line and url
  -- not all repos have master branch, thus link might be broken!
  CONCAT("https://github.com/", repo_name, "/blob/master/", path) as link,
  repo_name,
  path
FROM (
  SELECT
    distinct_url_line_2.id as id,
    distinct_url_line_2.url as url,
    distinct_url_line_2.line as line,
    distinct_url_line_2.repo_name as repo_name,
    MIN(all_2.path) as path
  FROM (
      SELECT
        distinct_url_line.id as id,
        distinct_url_line.url as url,
        distinct_url_line.line as line,
        MIN(all.repo_name) as repo_name
      FROM (
          SELECT
            MIN(id) as id, url, line
          FROM
            [snippets.c_q]
          GROUP BY
            url, line
      ) as distinct_url_line
      LEFT JOIN [snippets.c_q] as all
      ON all.id = distinct_url_line.id
        AND all.url = distinct_url_line.url
        AND all.line = distinct_url_line.line
      GROUP BY
        id, url, line
  ) as distinct_url_line_2
  LEFT JOIN [snippets.c_q] as all_2
  ON all_2.id = distinct_url_line_2.id
    AND all_2.url = distinct_url_line_2.url
    AND all_2.line = distinct_url_line_2.line
    AND all_2.repo_name = distinct_url_line_2.repo_name
  GROUP BY
    id, url, line, repo_name
);

=> c_q_distinct_url_line


--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
----------------------------------------------
--- C-specific (header vs. implementation) ---
----------------------------------------------
SELECT
  id,
  url,
  line,
  repo_name,
  path,
  LOWER(REGEXP_EXTRACT(path, r'(\.[^.]+$)')) as file_ext
FROM
  [snippets.c] s;
  
=> c_file_ext


SELECT
  id,
  url,
  line,
  repo_name,
  path,
  LOWER(REGEXP_EXTRACT(path, r'(\.[^.]+$)')) as file_ext
FROM
  [snippets.c_a_q_distinct_url_file] s;

=> c_a_q_distinct_url_file_file_ext


SELECT
  id,
  url,
  line,
  repo_name,
  path,
  LOWER(REGEXP_EXTRACT(path, r'(\.[^.]+$)')) as file_ext
FROM
  [snippets.c_q_distinct_url_file] s;

=> c_q_distinct_url_file_file_ext


-------------------------------------------------------------------------------------------------
--- CREATE TABLE WITH DISTINCT SO-URLS (NOT A OR Q) PER DISTINCT FILES (SAME FILE ID AND URL) ---
-------------------------------------------------------------------------------------------------
SELECT
  id, url, line,
  -- not all repos have master branch, thus link might be broken!
  CONCAT("https://github.com/", repo_name, "/blob/master/", path) as link,
  repo_name, path
FROM (
  SELECT
    distinct_id_url_2.id as id,
    distinct_id_url_2.url as url,
    distinct_id_url_2.repo_name as repo_name,
    distinct_id_url_2.path as path,
    MIN(all_2.line) as line
  FROM (
    SELECT
      distinct_id_url.id as id,
      distinct_id_url.url as url,
      distinct_id_url.repo_name as repo_name,
      MIN(all.path) as path
      FROM (
        SELECT
          id, url, MIN(repo_name) as repo_name
        FROM
          [snippets.c_other]
        GROUP BY
          id, url
      ) as distinct_id_url
      LEFT JOIN [snippets.c_other] as all
        ON all.id = distinct_id_url.id
        AND all.url = distinct_id_url.url
        AND all.repo_name = distinct_id_url.repo_name 
    GROUP BY id, url, repo_name
  ) as distinct_id_url_2
  LEFT JOIN [snippets.c_other] as all_2
  ON all_2.id = distinct_id_url_2.id
    AND all_2.url = distinct_id_url_2.url
    AND all_2.repo_name = distinct_id_url_2.repo_name
    AND all_2.path = distinct_id_url_2.path
  GROUP BY id, url, repo_name, path
); 
  
=> c_other_distinct_url_file


SELECT
  id,
  url,
  line,
  repo_name,
  path,
  LOWER(REGEXP_EXTRACT(path, r'(\.[^.]+$)')) as file_ext
FROM
  [snippets.c_other_distinct_url_file] s;

=> c_other_distinct_url_file_file_ext