
-- Status: 20.07.2016

-----------------------
--- ANALYZE SO-URLS ---
-----------------------

-- total number of references
SELECT COUNT(*)
FROM [snippets.cs];

=> 30212

-- number of references to answers or questions
SELECT COUNT(*)
FROM [snippets.cs_a_q];

=> 30004

-- number of references to answers
SELECT COUNT(*)
FROM [snippets.cs_a_q]
WHERE
  REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/a\/[\d]+)');
  
=> 11843

-- number of references to questions
SELECT COUNT(*)
FROM [snippets.cs_a_q]
WHERE
  REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/q\/[\d]+)');
  
=> 18161

-- number of other references
SELECT COUNT(*)
FROM [snippets.cs_other];
   
=> 208

----------------------------------------------------------------------------------------------

-- number of distinct references to answers or questions (counting only distinct urls per file)
SELECT COUNT(*)
FROM [snippets.cs_a_q_distinct_url_file];

=> 13169 

-- number of distinct files with reference to answer or question
SELECT COUNT(*)
FROM (
  SELECT id
  FROM [snippets.cs_a_q_distinct_url_file]
  GROUP BY id
);

=> 10508

-- number of distinct references to threads
SELECT COUNT(*)
FROM [snippets.cs_q_distinct_url_file];

=> 13141

-- number of distinct referenced threads
SELECT COUNT(*)
FROM (
  SELECT url
  FROM [snippets.cs_q_distinct_url_file]
  GROUP BY url
);

=> 4810

-- number of distinct references to answers
SELECT COUNT(*)
FROM [snippets.cs_a_q_distinct_url_file]
WHERE
  REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/a\/[\d]+)');

=> 5778

-- number of distinct references to questions
SELECT COUNT(*)
FROM [snippets.cs_a_q_distinct_url_file]
WHERE
  REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/q\/[\d]+)');

=> 7391

-- number of distinct references to other urls
SELECT COUNT(*)
FROM (
  SELECT id, url
  FROM [snippets.cs_other]
  GROUP BY id, url
);

=> 136

----------------------------------------------------------------------------------------------
 
-- number of distinct URLS
SELECT COUNT(*)
FROM (
  SELECT url
  FROM [snippets.cs]
  GROUP BY url
);

=> 5423


-- number of distinct answer URLS
SELECT COUNT(*)
FROM (
  SELECT url
  FROM [snippets.cs_a_q]
  WHERE REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/a\/[\d]+)')
  GROUP BY url
);

=> 1492

-- number of distinct question URLS
SELECT COUNT(*)
FROM (
  SELECT url
  FROM [snippets.cs_a_q]
  WHERE REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/q\/[\d]+)')
  GROUP BY url
);

=> 3834

-- number of distinct other URLS
SELECT COUNT(*)
FROM (
  SELECT url
  FROM [snippets.cs_other]
  GROUP BY url
);

=> 97

-- analyze other references
SELECT *
FROM [snippets.cs_other]
WHERE
  NOT REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/users\/[^\s]*)') -- links to users
  AND NOT REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/(?:questions\/)?(?:tags|tagged)\/[^\s]*)') -- links to tags
  AND NOT REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/search\?[^\s]*)') -- links to SO search
  AND NOT REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/feeds\/[^\s]*)') -- links to RSS feed
  AND NOT REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/posts\/[^\s]*)') -- links to revision of post
  AND NOT REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/(?:q|questions)\/?[^\s]*)') -- possibly line break
  AND NOT REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/?$)'); -- links to SO root

----------------------------------------------------------------------------------------------

-- 25.08.2016

-- repos with reference to answer or question
SELECT COUNT(*)
FROM (
  SELECT repo_name
  FROM [snippets.cs_a_q_distinct_url_file]
  GROUP BY repo_name
);

=> 5172

-- total number of repos containing c# files
SELECT COUNT(*)
FROM (
  SELECT repo_name
  FROM [snippets.repos_file_ext]
  WHERE file_ext=".cs"
  GROUP BY repo_name
);

=> 116587

