
-- Status: 20.07.2016

-----------------------
--- ANALYZE SO-URLS ---
-----------------------

-- total number of references
SELECT COUNT(*)
FROM [snippets.js];

=> 3674298

-- number of references to answers or questions
SELECT COUNT(*)
FROM [snippets.js_a_q];

=> 3603417

-- number of references to answers
SELECT COUNT(*)
FROM [snippets.js_a_q]
WHERE
  REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/a\/[\d]+)');
  
=> 980722

-- number of references to questions
SELECT COUNT(*)
FROM [snippets.js_a_q]
WHERE
  REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/q\/[\d]+)');
  
=> 2622695

-- number of other references
SELECT COUNT(*)
FROM [snippets.js_other];
   
=> 70881

----------------------------------------------------------------------------------------------

-- number of distinct references to answers or questions (counting only distinct urls per file)
SELECT COUNT(*)
FROM [snippets.js_a_q_distinct_url_file];

=> 89178 

-- number of distinct files with reference to answer or question
SELECT COUNT(*)
FROM (
  SELECT id
  FROM [snippets.js_a_q_distinct_url_file]
  GROUP BY id
);

=> 62618

-- number of distinct references to threads
SELECT COUNT(*)
FROM [snippets.js_q_distinct_url_file];

=> 89075

-- number of distinct referenced threads
SELECT COUNT(*)
FROM (
  SELECT url
  FROM [snippets.js_q_distinct_url_file]
  GROUP BY url
);

=> 10044

-- number of distinct references to answers
SELECT COUNT(*)
FROM [snippets.js_a_q_distinct_url_file]
WHERE
  REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/a\/[\d]+)');

 => 28621

-- number of distinct references to questions
SELECT COUNT(*)
FROM [snippets.js_a_q_distinct_url_file]
WHERE
  REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/q\/[\d]+)');

 => 60557

-- number of distinct references to other urls
SELECT COUNT(*)
FROM (
  SELECT id, url
  FROM [snippets.js_other]
  GROUP BY id, url
);

=> 1769

----------------------------------------------------------------------------------------------
 
-- number of distinct URLS
SELECT COUNT(*)
FROM (
  SELECT url
  FROM [snippets.js]
  GROUP BY url
);

=> 12462


-- number of distinct answer URLS
SELECT COUNT(*)
FROM (
  SELECT url
  FROM [snippets.js_a_q]
  WHERE REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/a\/[\d]+)')
  GROUP BY url
);

=> 4066

-- number of distinct question URLS
SELECT COUNT(*)
FROM (
  SELECT url
  FROM [snippets.js_a_q]
  WHERE REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/q\/[\d]+)')
  GROUP BY url
);

=> 8003

-- number of distinct other URLS
SELECT COUNT(*)
FROM (
  SELECT url
  FROM [snippets.js_other]
  GROUP BY url
);

=> 393

-- analyze other references
SELECT *
FROM [snippets.js_other]
WHERE
  NOT REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/users\/[^\s]*)') -- links to users
  AND NOT REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/(?:questions\/)?(?:tags|tagged)\/[^\s]*)') -- links to tags
  AND NOT REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/search\?[^\s]*)') -- links to SO search
  AND NOT REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/feeds\/[^\s]*)') -- links to RSS feed
  AND NOT REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/posts\/[^\s]*)') -- links to revision of post
  AND NOT REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/(?:q|questions)\/?[^\s]*)') -- possibly line break
  AND NOT REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/?$)'); -- links to SO root

----------------------------------------------------------------------------------------------

-- 25.08.2016

-- repos with reference to answer or question
SELECT COUNT(*)
FROM (
  SELECT repo_name
  FROM [snippets.js_a_q_distinct_url_file]
  GROUP BY repo_name
);

=> 36291

-- total number of repos containing js files
SELECT COUNT(*)
FROM (
  SELECT repo_name
  FROM [snippets.repos_file_ext]
  WHERE file_ext=".js"
  GROUP BY repo_name
);

=> 1033292

