
-- Status: 20.07.2016

-----------------------
--- ANALYZE SO-URLS ---
-----------------------

-- total number of references
SELECT COUNT(*)
FROM [snippets.php];

=> 74549

-- number of references to answers or questions
SELECT COUNT(*)
FROM [snippets.php_a_q];

=> 73329

-- number of references to answers
SELECT COUNT(*)
FROM [snippets.php_a_q]
WHERE
  REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/a\/[\d]+)');
  
=> 19162

-- number of references to questions
SELECT COUNT(*)
FROM [snippets.php_a_q]
WHERE
  REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/q\/[\d]+)');
  
=> 54167

-- number of other references
SELECT COUNT(*)
FROM [snippets.php_other];
   
=> 1220

----------------------------------------------------------------------------------------------

-- number of distinct references to answers or questions (counting only distinct urls per file)
SELECT COUNT(*)
FROM [snippets.php_a_q_distinct_url_file];

=> 13493 

-- number of distinct with files reference to answer or question
SELECT COUNT(*)
FROM (
  SELECT id
  FROM [snippets.php_a_q_distinct_url_file]
  GROUP BY id
);

=> 11950

-- number of distinct references to threads
SELECT COUNT(*)
FROM [snippets.php_q_distinct_url_file];

=> 13466

-- number of distinct referenced threads
SELECT COUNT(*)
FROM (
  SELECT url
  FROM [snippets.php_q_distinct_url_file]
  GROUP BY url
);

=> 3792

-- number of distinct references to answers
SELECT COUNT(*)
FROM [snippets.php_a_q_distinct_url_file]
WHERE
  REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/a\/[\d]+)');

 => 3185

-- number of distinct references to questions
SELECT COUNT(*)
FROM [snippets.php_a_q_distinct_url_file]
WHERE
  REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/q\/[\d]+)');

 => 10308

-- number of distinct references to other urls
SELECT COUNT(*)
FROM (
  SELECT id, url
  FROM [snippets.php_other]
  GROUP BY id, url
);

=> 406

----------------------------------------------------------------------------------------------
 
-- number of distinct URLS
SELECT COUNT(*)
FROM (
  SELECT url
  FROM [snippets.php]
  GROUP BY url
);

=> 4476


-- number of distinct answer URLS
SELECT COUNT(*)
FROM (
  SELECT url
  FROM [snippets.php_a_q]
  WHERE REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/a\/[\d]+)')
  GROUP BY url
);

=> 1320

-- number of distinct question URLS
SELECT COUNT(*)
FROM (
  SELECT url
  FROM [snippets.php_a_q]
  WHERE REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/q\/[\d]+)')
  GROUP BY url
);

=> 2967

-- number of distinct other URLS
SELECT COUNT(*)
FROM (
  SELECT url
  FROM [snippets.php_other]
  GROUP BY url
);

=> 189

-- analyze other references
SELECT *
FROM [snippets.php_other]
WHERE
  NOT REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/users\/[^\s]*)') -- links to users
  AND NOT REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/(?:questions\/)?(?:tags|tagged)\/[^\s]*)') -- links to tags
  AND NOT REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/search\?[^\s]*)') -- links to SO search
  AND NOT REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/feeds\/[^\s]*)') -- links to RSS feed
  AND NOT REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/posts\/[^\s]*)') -- links to revision of post
  AND NOT REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/(?:q|questions)\/?[^\s]*)') -- possibly line break
  AND NOT REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/?$)'); -- links to SO root

----------------------------------------------------------------------------------------------

-- 25.08.2016

-- repos with reference to answer or question
SELECT COUNT(*)
FROM (
  SELECT repo_name
  FROM [snippets.php_a_q_distinct_url_file]
  GROUP BY repo_name
);

=> 8067

-- total number of repos containing php files
SELECT COUNT(*)
FROM (
  SELECT repo_name
  FROM [snippets.repos_file_ext]
  WHERE file_ext=".php"
  GROUP BY repo_name
);

=> 318473

