
-- Status: 20.07.2016

-----------------------
--- ANALYZE SO-URLS ---
-----------------------

-- total number of references
SELECT COUNT(*)
FROM [snippets.py];

=> 93774

-- number of references to answers or questions
SELECT COUNT(*)
FROM [snippets.py_a_q];

=> 92123

-- number of references to answers
SELECT COUNT(*)
FROM [snippets.py_a_q]
WHERE
  REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/a\/[\d]+)');
  
=> 30643

-- number of references to questions
SELECT COUNT(*)
FROM [snippets.py_a_q]
WHERE
  REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/q\/[\d]+)');
  
=> 61480

-- number of other references
SELECT COUNT(*)
FROM [snippets.py_other];
   
=> 1651

----------------------------------------------------------------------------------------------

-- number of distinct references to answers or questions (counting only distinct urls per file)
SELECT COUNT(*)
FROM [snippets.py_a_q_distinct_url_file];

=> 30026 

-- number of distinct files with reference to answer or question
SELECT COUNT(*)
FROM (
  SELECT id
  FROM [snippets.py_a_q_distinct_url_file]
  GROUP BY id
);

=> 25202

-- number of distinct references to threads
SELECT COUNT(*)
FROM [snippets.py_q_distinct_url_file];

=> 29894

-- number of distinct referenced threads
SELECT COUNT(*)
FROM (
  SELECT url
  FROM [snippets.py_q_distinct_url_file]
  GROUP BY url
);

=> 9667

-- number of distinct references to answers
SELECT COUNT(*)
FROM [snippets.py_a_q_distinct_url_file]
WHERE
  REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/a\/[\d]+)');

 => 8599

-- number of distinct references to questions
SELECT COUNT(*)
FROM [snippets.py_a_q_distinct_url_file]
WHERE
  REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/q\/[\d]+)');

 => 21427

-- number of distinct references to other urls
SELECT COUNT(*)
FROM (
  SELECT id, url
  FROM [snippets.py_other]
  GROUP BY id, url
);

=> 530

----------------------------------------------------------------------------------------------
 
-- number of distinct URLS
SELECT COUNT(*)
FROM (
  SELECT url
  FROM [snippets.py]
  GROUP BY url
);

=> 11485


-- number of distinct answer URLS
SELECT COUNT(*)
FROM (
  SELECT url
  FROM [snippets.py_a_q]
  WHERE REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/a\/[\d]+)')
  GROUP BY url
);

=> 3357

-- number of distinct question URLS
SELECT COUNT(*)
FROM (
  SELECT url
  FROM [snippets.py_a_q]
  WHERE REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/q\/[\d]+)')
  GROUP BY url
);

=> 7876

-- number of distinct other URLS
SELECT COUNT(*)
FROM (
  SELECT url
  FROM [snippets.py_other]
  GROUP BY url
);

=> 252

-- analyze other references
SELECT *
FROM [snippets.py_other]
WHERE
  NOT REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/users\/[^\s]*)') -- links to users
  AND NOT REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/(?:questions\/)?(?:tags|tagged)\/[^\s]*)') -- links to tags
  AND NOT REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/search\?[^\s]*)') -- links to SO search
  AND NOT REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/feeds\/[^\s]*)') -- links to RSS feed
  AND NOT REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/posts\/[^\s]*)') -- links to revision of post
  AND NOT REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/(?:q|questions)\/?[^\s]*)') -- possibly line break
  AND NOT REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/?$)'); -- links to SO root

----------------------------------------------------------------------------------------------

-- 25.08.2016

-- repos with reference to answer or question
SELECT COUNT(*)
FROM (
  SELECT repo_name
  FROM [snippets.py_a_q_distinct_url_file]
  GROUP BY repo_name
);

=> 16711

-- total number of repos containing python files
SELECT COUNT(*)
FROM (
  SELECT repo_name
  FROM [snippets.repos_file_ext]
  WHERE file_ext=".py"
  GROUP BY repo_name
);

=> 491232

