
-- Status: 10.08.2016

-----------------------
--- ANALYZE SO-URLS ---
-----------------------

-- total number of references
SELECT COUNT(*)
FROM [snippets.r];

=> 1979

-- number of references to answers or questions
SELECT COUNT(*)
FROM [snippets.r_a_q];

=> 1941

-- number of references to answers
SELECT COUNT(*)
FROM [snippets.r_a_q]
WHERE
  REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/a\/[\d]+)');
  
=> 378

-- number of references to questions
SELECT COUNT(*)
FROM [snippets.r_a_q]
WHERE
  REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/q\/[\d]+)');
  
=> 1563

-- number of other references
SELECT COUNT(*)
FROM [snippets.r_other];
   
=> 38

----------------------------------------------------------------------------------------------

-- number of distinct references to answers or questions (counting only distinct urls per file)
SELECT COUNT(*)
FROM [snippets.r_a_q_distinct_url_file];

=> 1472 

-- number of distinct files with reference to answer or question
SELECT COUNT(*)
FROM (
  SELECT id
  FROM [snippets.r_a_q_distinct_url_file]
  GROUP BY id
);

=> 1107

-- number of distinct references to threads
SELECT COUNT(*)
FROM [snippets.r_q_distinct_url_file];

=> 1469

-- number of distinct referenced threads
SELECT COUNT(*)
FROM (
  SELECT url
  FROM [snippets.r_q_distinct_url_file]
  GROUP BY url
);

=> 912

-- number of distinct references to answers
SELECT COUNT(*)
FROM [snippets.r_a_q_distinct_url_file]
WHERE
  REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/a\/[\d]+)');

 => 287

-- number of distinct references to questions
SELECT COUNT(*)
FROM [snippets.r_a_q_distinct_url_file]
WHERE
  REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/q\/[\d]+)');

 => 1185

-- number of distinct references to other urls
SELECT COUNT(*)
FROM (
  SELECT id, url
  FROM [snippets.r_other]
  GROUP BY id, url
);

=> 31

----------------------------------------------------------------------------------------------
 
-- number of distinct URLS
SELECT COUNT(*)
FROM (
  SELECT url
  FROM [snippets.r]
  GROUP BY url
);

=> 984

-- number of distinct answer URLS
SELECT COUNT(*)
FROM (
  SELECT url
  FROM [snippets.r_a_q]
  WHERE REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/a\/[\d]+)')
  GROUP BY url
);

=> 202

-- number of distinct question URLS
SELECT COUNT(*)
FROM (
  SELECT url
  FROM [snippets.r_a_q]
  WHERE REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/q\/[\d]+)')
  GROUP BY url
);

=> 760

-- number of distinct other URLS
SELECT COUNT(*)
FROM (
  SELECT url
  FROM [snippets.r_other]
  GROUP BY url
);

=> 22

-- analyze other references
SELECT *
FROM [snippets.r_other]
WHERE
  NOT REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/users\/[^\s]*)') -- links to users
  AND NOT REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/(?:questions\/)?(?:tags|tagged)\/[^\s]*)') -- links to tags
  AND NOT REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/search\?[^\s]*)') -- links to SO search
  AND NOT REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/feeds\/[^\s]*)') -- links to RSS feed
  AND NOT REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/posts\/[^\s]*)') -- links to revision of post
  AND NOT REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/(?:q|questions)\/?[^\s]*)') -- possibly line break
  AND NOT REGEXP_MATCH(url, r'(https?:\/\/stackoverflow\.com\/?$)'); -- links to SO root

----------------------------------------------------------------------------------------------

-- 25.08.2016

-- repos with reference to answer or question
SELECT COUNT(*)
FROM (
  SELECT repo_name
  FROM [snippets.r_a_q_distinct_url_file]
  GROUP BY repo_name
);

=> 668

-- total number of repos containing python files
SELECT COUNT(*)
FROM (
  SELECT repo_name
  FROM [snippets.repos_file_ext]
  WHERE file_ext=".r"
  GROUP BY repo_name
);

=> 23206

